package provide nestedbox 1.0

# nestedbox.tcl
#	This package displays trees as a set of nested boxes.
#	I didn't find any literature references to this technique, but I
#	didn't look hard.  I may have just invented it.
#
# Copyright (c) 1998 Clif Flynt 
#
# This program is free for use in non-commercial applications, provided 
# that the copyright information is retained.  It may be licensed from
# Flynt Consulting Services for use in commercial applications.    
#
#  This code is described in "Tcl/Tk for Real Programmers", 
#  published by Academic Press Professional
#  ISBN: 0122612051 
#

namespace eval nestedbox {
namespace export DisplayTreeInBoxes  BoxLayout 

variable namespaceparent [namespace parent]

################################################################
# proc BoxLayout {node}--
#    Layouts nodes for a boxed display algorithm where
#    all nodes are shown as raised rectangles over their parents.
# Arguments
#   node:	The base node to work from.  This is recursive.
# 
# Results
#   Sets Xoffset and Yoffset
#   Sets height/width if they aren't already defined.
# 
proc BoxLayout {node} {
    set clst [$node childList]
    if {[llength $clst] == 0} {return} 
    
    # mywidth is the width in pixels necessary for this node & children
    set mywidth [BoxLayoutChildrenWidth $node -r]
    set my0wd [expr [$node get width] + 2* [$node get xpad]]


    set horizontal 0;
    foreach c $clst {
        BoxLayout $c
	if {[llength [$c childList]] > 0} {set horizontal 1} 
    }

    set c0 [lindex $clst 0]
    set clst [lreplace $clst 0 0]
    set c0width [BoxLayoutChildrenWidth $c0 -r]
    
    set sibLst [$node siblingList] 

    set ht [$node get height]

    foreach s $sibLst {
        if {$ht < [$s get height]} {set ht [$s get height]}
    }

    if {[llength $clst] == 0} {
    $c0 set -node Xoffset 0
# puts "$c0 XOFF0: $mywidth $c0width"
    } else {
#    set xoff [expr  $mywidth/2 - $c0width/2]
#puts "$c0 XOFF: $xoff  $mywidth $c0width"
#    $c0 set -node Xoffset $xoff
   
    set c0cwd [expr [$c0 get width] + 2* [$c0 get xpad]]
    set xoff [expr  $c0width/2 - $mywidth/2 - ($c0cwd/2 - $my0wd/2)]
    $c0 set -node Xoffset $xoff
    }

    $c0 set -node Yoffset [expr [$node get height] + [$node get ypad]]
    
    foreach c $clst {
	set cwd1 [BoxLayoutChildrenWidth $c -r]
        set cw [expr $cwd1 - [$c get width]]
        if {$horizontal} {
            $c set Yoffset 0
	    $c set Xoffset [expr $cw + $c0width + [$c get xpad] + [$c get xIpad]]
 #puts "$c: Xoffset: [$c get Xoffset] $cw $c0width"
	} else {
	    $c set Xoffset 0
	    $c set Yoffset [expr [$c0 get height] + [$c get ypad] + [$c get yIpad]]
	}
	set c0 $c
	set c0width $cwd1
    }
#puts "C0X: $c0 [$c0 get Xoffset] "
}
################################################################
# proc BoxLayoutChildrenWidth {node {recurs ""}}--
# Returns the width of this node + children in pixels
#    
# Arguments
#   node:	The base node to get a width of
#   ?-r?:	Recurse and get widths of kids.
# Results
#   Returns the width based on the algorithm that 
#   if any children have child nodes, the nodes will be arranged horizontally
#   else the child nodes will be arranged vertically.

proc BoxLayoutChildrenWidth {node {recurs ""}} {
    variable namespaceparent 
    if {[$node get height] == 0} {
	eval ${namespaceparent}::CanvasTreeSetSize $node
    }
    
    ;# Width = xpad + the width of the rectangle outline.
    set width [expr [$node get xpad] * 2 +4 ]

    set clst [$node childList]
    set haschildren 0;

    foreach c $clst {
	if {[llength [$c childList]] > 0} {
	    set haschildren 1;
	}
    }

    foreach c $clst {
        if {[$c get height] == 0} {
	eval ${namespaceparent}::CanvasTreeSetSize $c
        }
	if {[llength [$c childList]] > 0} {
            if {[string match $recurs "-r"]} {
	        set w [BoxLayoutChildrenWidth $c -r]
	    } else {
	        set w [$c get width]
	    }
	    incr width $w
	} else {
            set w [expr [$c get width] + 2* [$c get xpad] +4]
	    if {$haschildren} {
	        incr width $w
	    } else {
	    if {$w > $width} {
	        set width $w
            }
	    }
       }
  }
  
  set w [expr [$node get width] + 2* [$node get xpad] +4 ]
  if {$w > $width} {
      set width $w
  }

  if {([llength $clst] > 0) && ($haschildren == 0)} {incr width 10}

#        set x [info level]
#        incr x -1;
#        set y [info level $x]  
#  puts "NODE: $node WIDTH: $width  has: $haschildren recurse: $recurs : $y"
  return $width
}


################################################################
# proc makeRect {node cvs}--
#    Makes a rectangle around the area described in the BBOX boundingbox
#
# Arguments
#   node:	The node to get BBOX from
#   cvs		Canvas to draw to.
# Results
#   Draws a box with tag $node.box, and lowers it.
# 
proc makeRect {node cvs} {
    variable namespaceparent 
;#    foreach i [$node childList] {
;#	makeRect $i $cvs 
;#    }
    set bbox [$node get BBOX]

    set xpad [$node get xIpad]
    set ypad [$node get yIpad]

    set bbox [lreplace $bbox 0 0 [expr [lindex $bbox 0] -$xpad]]
    set bbox [lreplace $bbox 1 1 [expr [lindex $bbox 1] -$ypad]]
    set bbox [lreplace $bbox 2 2 [expr [lindex $bbox 2] +$xpad]]
    set bbox [lreplace $bbox 3 3 [expr [lindex $bbox 3] +$xpad]]


    eval ${namespaceparent}::raisedRect $cvs [lindex $bbox 0] [lindex $bbox 1] \
      [expr [lindex $bbox 2] - [lindex $bbox 0]] \
      [expr [lindex $bbox 3] - [lindex $bbox 1]] \
      -tag $node.box
    $cvs addtag $node withtag $node.box
    $cvs lower $node.box
}

################################################################
# proc DisplayTreeInBoxes {node cvs args}--
#    Displays a tree in raised boxes.
#    Expects layout and x/y DesiredPositions to be set
#    May draw recursively if "-r" set
# Arguments
#   node:	The ndoe to start from
#   cvs		The canvas to draw upon
#   args	optional arguments. -r for recursive.
# Results
#   
# 
proc DisplayTreeInBoxes {node cvs args} {
    variable namespaceparent 
# puts "DispinBox: $node -- $args"
    if {[set pos [lsearch $args "-r" ]] >= 0} {
        set recurs 1
	set args [lreplace $args $pos $pos]
    } else {
        set recurs 0
    }

    eval ${namespaceparent}::drawText $node $cvs $args
    
    if {$recurs} {
    foreach i [$node childList] {
	eval DisplayTreeInBoxes $i $cvs -r $args
    }
    }
    CanvasTreeSetBBOX $node $cvs
    makeRect $node $cvs 
    $cvs raise $node.txt
}

################################################################
# proc CanvasTreeSetBBOX {node cvs }--
#    Sets the BBOX variable/ID for the size of the bounding box
#    of this node and all children.
# Arguments
#   node:	The node (and children) to check.
#   cvs:	The canvas this node lives in
# 
# Results
#   Sets a variable in the Node.
# 
proc CanvasTreeSetBBOX {node cvs} {
    set bb [$cvs bbox $node]
    
    if {[string match $bb ""]} {set bb {0 0 0 0}}

    ;# puts "$node: START: $bb"
    foreach n [$node childList] {
	CanvasTreeSetBBOX $n $cvs
	set cbb [$n get BBOX]
	for {set i 0} {$i < 2} {incr i} {
	    if {[lindex $bb $i] > [lindex $cbb $i]} {
		set bb [lreplace $bb $i $i [lindex $cbb $i]]
	    }
	}
	for {set i 2} {$i < 4} {incr i} {
	    if {[lindex $bb $i] < [lindex $cbb $i]} {
		set bb [lreplace $bb $i $i [lindex $cbb $i]]
	    }
	}
    }
    
    for {set i 0} {$i < 4} {incr i} {
        set bb [lreplace $bb $i $i [expr [lindex $bb $i] -1]]
    }
#    set bb [lreplace $bb 1 1 [expr [lindex $bb 1] -2]]
        
    $node set -node BBOX $bb
#        set x [info level]
#        incr x -1;
#        set y [info level $x]  
#   puts "CanvasTreeSetBBOX: $node $bb -- $y"
}


}
