# showTreeTest.tcl
#	An example of using the canvastree.  Can force various displays
#	with arguments.
#
# Copyright (c) 1998 Clif Flynt 
#
# This program is free for use in non-commercial applications, provided 
# that the copyright information is retained.  It may be licensed from
# Flynt Consulting Services for use in commercial applications.    
#
#  This code is described in "Tcl/Tk for Real Programmers", 
#  published by Academic Press Professional
#  ISBN: 0122612051 
#

lappend auto_path "."

package require canvastree

################################################################
# proc testLayout {{vertical ""}} 
#     tests the Moen layout code with a slightly complex tree.
# Arguments
#   vertical - selects vertical or horizontal layout
# Results
#   Displays a screen.
# 
proc testLayout {args} {

    set base [newCanvasTree]
    $base set -node displayText [list "base" "one child" $base]
    $base set -tree parent_distance 5
    $base set -node renderProc raisedTextRenderNode
    $base set -node Font [list times 16 bold]

    if {[string first "-vertical" $args] >= 0} {
	$base set -tree layout vertical
    }

    set ch1 [$base add]
    $ch1 set -node displayText [list "child 1" "3 kids" $ch1]
    $ch1 set -node renderProc raisedTextRenderNode

    set ch1_1 [$ch1 add]
    $ch1_1 set -node displayText [list "child 1.1" $ch1_1]
    $ch1_1 set -node renderProc raisedTextRenderNode

    set ch1_2 [$ch1 add]
    $ch1_2 set -node displayText [list "child 1.2" $ch1_2]

    set ch1_3 [$ch1 add]
    $ch1_3 set -node displayText [list "child 1.3" $ch1_3]

    set ch1_1_1 [$ch1_1 add]
    $ch1_1_1 set -node displayText [list "child 1.1.1" " two lines" $ch1_1_1]

    set ch1_1_2 [$ch1_1 add]
    $ch1_1_2 set -node displayText [list "child 1.1.2" $ch1_1_2]

    set ch1_1_3 [$ch1_1 add]
    $ch1_1_3 set -node displayText [list "child 1.1.3" $ch1_1_3]

    set ch1_2_1 [$ch1_2 add]
    $ch1_2_1 set -node displayText [list "child 1.2.1" $ch1_2_1]

    set ch1_2_2 [$ch1_2 add]
    $ch1_2_2 set -node displayText [list "child 1.2.2" $ch1_2_2]

    set ch1_2_2_1 [$ch1_2_2 add]
    $ch1_2_2_1 set -node displayText [list "child 1.2.2.1" $ch1_2_2_1]

    set ch1_2_2_2 [$ch1_2_2 add]
    $ch1_2_2_2 set -node displayText [list "child 1.2.2.2" $ch1_2_2_2]

    if {[string first "-vertical" $args] >= 0} {
	$base set -node yDesiredPosition 20
	$base set -node xDesiredPosition 350
	canvas .c -width 800 -height 600
    } else {
	$base set -node yDesiredPosition 400
	$base set -node xDesiredPosition 50
	canvas .c -width 700 -height 600
    }


    #    set lst [$base dump -r]

    #    foreach l $lst {

    #	puts "$l"

    #    }


    pack .c


    if {[string first "-boxed" $args] < 0} {
	canvastree::moenLayout $base
        if {[string first "-vertical" $args] >= 0} {
	    $base set -node yDesiredPosition 20
	    $base set -node xDesiredPosition 400
	} else {
	    $base set -node yDesiredPosition 350
	    $base set -node xDesiredPosition 60
	}
	canvastree::setDesiredPositions $base
	$base display .c -r
    } else {
	canvastree::BoxLayout $base
	$base set -node yDesiredPosition 20
	$base set -node xDesiredPosition 350
	canvastree::setDesiredPositions $base
	canvastree::DisplayTreeInBoxes $base .c -r
    }


    #    drawContour .c $base

    #    drawContour .c $ch1
    canvastree::drawContour .c $ch1_1

    #    drawContour .c $ch1_3

    #    drawContour .c $ch1_2


    return
}

testLayout $argv
